==> daebus-0.0.17.tar.gz: daebus-0.0.17/tests/test_integration.py <==
import unittest
import time
import json
import threading
import random
from unittest.mock import MagicMock, patch

from daebus import Daebus


class TestIntegration(unittest.TestCase):
    """Integration tests for Daebus with simulated Redis callbacks"""
    
    @patch('redis.Redis')
    def test_full_message_flow(self, mock_redis):
        """Test the complete message flow with simulated Redis callbacks"""
        
        # Results tracking
        results = {}
        results_lock = threading.Lock()
        
        # Create our app
        app = Daebus("integration_test")
        
        # Create a Redis mock
        redis_mock = mock_redis.return_value
        redis_mock.decode_responses = True
        
        # Store Redis responses that would be sent back
        redis_responses = {}
        
        # Mock Redis publish to capture and record responses
        def mock_publish(channel, data):
            with results_lock:
                # Decode the data if needed
                if isinstance(data, bytes):
                    data = data.decode('utf-8')
                
                # Store the response
                parsed_data = json.loads(data)
                redis_responses[channel] = parsed_data
                
                # For testing purposes, simulate message received callback
                if channel.startswith("reply_"):
                    # Extract proper message ID from channel - assuming format reply_[message_id]
                    message_id = channel.split("reply_")[1]
                    
                    # Store by message ID for easier verification
                    results[message_id] = parsed_data
            
            return 1  # Simulate 1 subscriber received the message
        
        # Set up the mock
        redis_mock.publish = mock_publish
        app.redis = redis_mock
        app._running = True
        app.thread_pool = app._create_thread_pool()
        
        # Define our action handlers
        @app.action("echo")
        def handle_echo():
            # Simple echo action that returns the payload
            payload = app.request.payload
            message_id = payload.get("id", "unknown")
            
            # Echo back the data
            response = app.response.success({
                "echo": payload,
                "id": message_id,
                "timestamp": time.time()
            })
            return response
        
        @app.action("compute")
        def handle_compute():
            # Action that performs a computation
            payload = app.request.payload
            message_id = payload.get("id", "unknown")
            
            # Get numbers to add
            numbers = payload.get("numbers", [])
            
            # Compute the sum
            result = sum(numbers)
            
            # Return the result
            response = app.response.success({
                "result": result,
                "id": message_id,
                "timestamp": time.time()
            })
            return response
        
        # Test with just 2 messages to simplify the test
        # 1 echo and 1 compute message
        messages = []
        
        # Add echo message
        message_id = "echo_test"
        message = {
            "action": "echo",
            "payload": {
                "id": message_id,
                "text": "This is a test message",
                "timestamp": time.time()
            },
            "reply_to": f"reply_{message_id}"  # Channel for responses
        }
        messages.append((message_id, message))
            
        # Add compute message
        message_id = "compute_test"
        message = {
            "action": "compute",
            "payload": {
                "id": message_id,
                "numbers": [10, 20, 30, 40, 50],
                "timestamp": time.time()
            },
            "reply_to": f"reply_{message_id}"  # Channel for responses
        }
        messages.append((message_id, message))
        
        # Process messages sequentially to avoid thread-local storage issues
        for message_id, message in messages:
            # Create a Redis-like message structure
            redis_message = {"data": json.dumps(message)}
            
            # Process message directly
            app._process_message(redis_message)
        
        # Short wait to ensure all processing completes
        time.sleep(0.5)
        
        # Verify all messages were processed and responses were sent
        self.assertEqual(len(results), 2, "Both messages should have responses")
        
        # Check echo response
        message_id = "echo_test"
        self.assertIn(message_id, results, f"Response for {message_id} should exist")
        self.assertIn("payload", results[message_id], "Response should have 'payload' field")
        self.assertIn("echo", results[message_id]["payload"], "Response payload should have 'echo' field")
        
        # Check the ID in the echo response
        echo_payload = results[message_id]["payload"]["echo"]
        self.assertEqual(echo_payload["id"], message_id, 
                        "Echo response should return the correct message ID")
        
        # Check compute response
        message_id = "compute_test"
        self.assertIn(message_id, results, f"Response for {message_id} should exist")
        self.assertIn("payload", results[message_id], "Response should have 'payload' field")
        self.assertIn("result", results[message_id]["payload"], "Response payload should have 'result' field")
        
        # Check the result value
        result_value = results[message_id]["payload"]["result"]
        self.assertEqual(result_value, 150, "Compute result should be 150")
        
        # Verify responses were sent to the correct channels
        for message_id, _ in messages:
            reply_channel = f"reply_{message_id}"
            self.assertIn(reply_channel, redis_responses,
                        f"Response should be sent to channel {reply_channel}")
        
        # Cleanup
        app.thread_pool.shutdown(wait=True)
    
    @patch('redis.Redis')
    def test_concurrent_service_communication(self, mock_redis):
        """Test concurrent communication between multiple services"""
        
        # Create a simpler test that doesn't rely on complex threading
        # Test each service individually first
        
        # Create Redis mock
        redis_mock = mock_redis.return_value
        redis_mock.decode_responses = True
        
        # Create the services
        service_a = Daebus("service_a", max_workers=2)
        service_b = Daebus("service_b", max_workers=2)
        service_c = Daebus("service_c", max_workers=2)
        
        # Initialize services
        for service in [service_a, service_b, service_c]:
            service.redis = redis_mock
            service._running = True
            service.thread_pool = service._create_thread_pool()
        
        # Results storage
        results = {}
        
        # Simplified mocking of Redis publish
        def mock_publish(channel, data):
            # Store result for later verification
            results[channel] = data
            return 1  # Simulate 1 recipient
            
        redis_mock.publish = mock_publish
        
        # Test service A directly
        @service_a.action("test_a")
        def handle_test_a():
            return service_a.response.success({"value": 42})
            
        # Create a message for service A
        message_a = {
            "action": "test_a",
            "payload": {"id": "test"},
            "reply_to": "reply_a"
        }
        
        # Send the message to service A
        service_a._process_message({"data": json.dumps(message_a)})
        
        # Verify the response
        self.assertIn("reply_a", results, "Service A should have sent a response")
        
        # Test service B directly
        @service_b.action("test_b")
        def handle_test_b():
            return service_b.response.success({"result": 84})
            
        # Create a message for service B
        message_b = {
            "action": "test_b",
            "payload": {"id": "test"},
            "reply_to": "reply_b"
        }
        
        # Send the message to service B
        service_b._process_message({"data": json.dumps(message_b)})
        
        # Verify the response
        self.assertIn("reply_b", results, "Service B should have sent a response")
        
        # Test service C directly
        @service_c.action("test_c")
        def handle_test_c():
            return service_c.response.success({"stored": True})
            
        # Create a message for service C
        message_c = {
            "action": "test_c",
            "payload": {"id": "test"},
            "reply_to": "reply_c"
        }
        
        # Send the message to service C
        service_c._process_message({"data": json.dumps(message_c)})
        
        # Verify the response
        self.assertIn("reply_c", results, "Service C should have sent a response")
        
        # Now test a simple end-to-end flow
        results.clear()  # Clear previous results
        
        # Set up proper handlers for the workflow
        @service_a.action("get_data")
        def a_handle_get_data():
            request_id = service_a.request.payload.get("id", "unknown")
            return service_a.response.success({
                "data": {"value": 42},
                "id": request_id
            })
            
        @service_b.action("process_data")
        def b_handle_process_data():
            request_id = service_b.request.payload.get("id", "unknown")
            value = service_b.request.payload.get("value", 0)
            result = value * 2
            return service_b.response.success({
                "result": result,
                "id": request_id
            })
            
        @service_c.action("store_result")
        def c_handle_store_result():
            request_id = service_c.request.payload.get("id", "unknown")
            result = service_c.request.payload.get("result", 0)
            return service_c.response.success({
                "stored": True,
                "id": request_id
            })
            
        # Create messages for a single workflow
        workflow_a = {
            "action": "get_data",
            "payload": {"id": "wf1"},
            "reply_to": "reply_wf1_a"
        }
        
        # Step 1: Process message A
        service_a._process_message({"data": json.dumps(workflow_a)})
        
        # Verify response from A
        self.assertIn("reply_wf1_a", results, "Service A should have sent a response")
        
        # Parse the response
        response_a = json.loads(results["reply_wf1_a"])
        self.assertEqual(response_a["payload"]["id"], "wf1", "Response should have the correct ID")
        
        # Get value from A's response
        value = response_a["payload"]["data"]["value"]
        
        # Step 2: Create and process message B
        workflow_b = {
            "action": "process_data",
            "payload": {"id": "wf1", "value": value},
            "reply_to": "reply_wf1_b"
        }
        
        service_b._process_message({"data": json.dumps(workflow_b)})
        
        # Verify response from B
        self.assertIn("reply_wf1_b", results, "Service B should have sent a response")
        
        # Parse the response
        response_b = json.loads(results["reply_wf1_b"])
        self.assertEqual(response_b["payload"]["id"], "wf1", "Response should have the correct ID")
        
        # Get result from B's response
        result = response_b["payload"]["result"]
        
        # Step 3: Create and process message C
        workflow_c = {
            "action": "store_result",
            "payload": {"id": "wf1", "result": result},
            "reply_to": "reply_wf1_c"
        }
        
        service_c._process_message({"data": json.dumps(workflow_c)})
        
        # Verify response from C
        self.assertIn("reply_wf1_c", results, "Service C should have sent a response")
        
        # Parse the response
        response_c = json.loads(results["reply_wf1_c"])
        self.assertEqual(response_c["payload"]["id"], "wf1", "Response should have the correct ID")
        
        # Verify the end-to-end data flow worked correctly
        self.assertEqual(result, value * 2, "Service B should have doubled the value")
        self.assertTrue(response_c["payload"]["stored"], "Service C should have confirmed storage")
        
        # Cleanup
        for service in [service_a, service_b, service_c]:
            service.thread_pool.shutdown(wait=True)


if __name__ == '__main__':
    unittest.main()

==> daebus-0.0.20.tar.gz: daebus-0.0.20/tests/test_integration.py <==
import unittest
import time
import json
import threading
import random
from unittest.mock import MagicMock, patch

from daebus import Daebus


class TestIntegration(unittest.TestCase):
    """Integration tests for Daebus with simulated Redis callbacks"""
    
    @patch('redis.Redis')
    def test_full_message_flow(self, mock_redis):
        """Test the complete message flow with simulated Redis callbacks"""
        
        # Results tracking
        results = {}
        results_lock = threading.Lock()
        
        # Create our app
        app = Daebus("integration_test")
        
        # Create a Redis mock
        redis_mock = mock_redis.return_value
        redis_mock.decode_responses = True
        
        # Store Redis responses that would be sent back
        redis_responses = {}
        
        # Mock Redis publish to capture and record responses
        def mock_publish(channel, data):
            with results_lock:
                # Decode the data if needed
                if isinstance(data, bytes):
                    data = data.decode('utf-8')
                
                # Store the response
                parsed_data = json.loads(data)
                redis_responses[channel] = parsed_data
                
                # For testing purposes, simulate message received callback
                if channel.startswith("reply_"):
                    # Extract proper message ID from channel - assuming format reply_[message_id]
                    message_id = channel.split("reply_")[1]
                    
                    # Store by message ID for easier verification
                    results[message_id] = parsed_data
            
            return 1  # Simulate 1 subscriber received the message
        
        # Set up the mock
        redis_mock.publish = mock_publish
        app.redis = redis_mock
        app._running = True
        app.thread_pool = app._create_thread_pool()
        
        # Define our action handlers
        @app.action("echo")
        def handle_echo():
            # Simple echo action that returns the payload
            payload = app.request.payload
            message_id = payload.get("id", "unknown")
            
            # Echo back the data
            response = app.response.send({
                "echo": payload,
                "id": message_id,
                "timestamp": time.time()
            })
            return response
        
        @app.action("compute")
        def handle_compute():
            # Action that performs a computation
            payload = app.request.payload
            message_id = payload.get("id", "unknown")
            
            # Get numbers to add
            numbers = payload.get("numbers", [])
            
            # Compute the sum
            result = sum(numbers)
            
            # Return the result
            response = app.response.send({
                "result": result,
                "id": message_id,
                "timestamp": time.time()
            })
            return response
        
        # Test with just 2 messages to simplify the test
        # 1 echo and 1 compute message
        messages = []
        
        # Add echo message
        message_id = "echo_test"
        message = {
            "action": "echo",
            "payload": {
                "id": message_id,
                "text": "This is a test message",
                "timestamp": time.time()
            },
            "reply_to": f"reply_{message_id}"  # Channel for responses
        }
        messages.append((message_id, message))
            
        # Add compute message
        message_id = "compute_test"
        message = {
            "action": "compute",
            "payload": {
                "id": message_id,
                "numbers": [10, 20, 30, 40, 50],
                "timestamp": time.time()
            },
            "reply_to": f"reply_{message_id}"  # Channel for responses
        }
        messages.append((message_id, message))
        
        # Process messages sequentially to avoid thread-local storage issues
        for message_id, message in messages:
            # Create a Redis-like message structure
            redis_message = {"data": json.dumps(message)}
            
            # Process message directly
            app._process_message(redis_message)
        
        # Short wait to ensure all processing completes
        time.sleep(0.5)
        
        # Verify all messages were processed and responses were sent
        self.assertEqual(len(results), 2, "Both messages should have responses")
        
        # Check echo response
        message_id = "echo_test"
        self.assertIn(message_id, results, f"Response for {message_id} should exist")
        self.assertIn("payload", results[message_id], "Response should have 'payload' field")
        self.assertIn("echo", results[message_id]["payload"], "Response payload should have 'echo' field")
        
        # Check the ID in the echo response
        echo_payload = results[message_id]["payload"]["echo"]
        self.assertEqual(echo_payload["id"], message_id, 
                        "Echo response should return the correct message ID")
        
        # Check compute response
        message_id = "compute_test"
        self.assertIn(message_id, results, f"Response for {message_id} should exist")
        self.assertIn("payload", results[message_id], "Response should have 'payload' field")
        self.assertIn("result", results[message_id]["payload"], "Response payload should have 'result' field")
        
        # Check the result value
        result_value = results[message_id]["payload"]["result"]
        self.assertEqual(result_value, 150, "Compute result should be 150")
        
        # Verify responses were sent to the correct channels
        for message_id, _ in messages:
            reply_channel = f"reply_{message_id}"
            self.assertIn(reply_channel, redis_responses,
                        f"Response should be sent to channel {reply_channel}")
        
        # Cleanup
        app.thread_pool.shutdown(wait=True)
    
    @patch('redis.Redis')
    def test_concurrent_service_communication(self, mock_redis):
        """Test concurrent communication between multiple services"""
        
        # Create a simpler test that doesn't rely on complex threading
        # Test each service individually first
        
        # Create Redis mock
        redis_mock = mock_redis.return_value
        redis_mock.decode_responses = True
        
        # Create the services
        service_a = Daebus("service_a", max_workers=2)
        service_b = Daebus("service_b", max_workers=2)
        service_c = Daebus("service_c", max_workers=2)
        
        # Initialize services
        for service in [service_a, service_b, service_c]:
            service.redis = redis_mock
            service._running = True
            service.thread_pool = service._create_thread_pool()
        
        # Results storage
        results = {}
        
        # Simplified mocking of Redis publish
        def mock_publish(channel, data):
            # Store result for later verification
            results[channel] = data
            return 1  # Simulate 1 recipient
            
        redis_mock.publish = mock_publish
        
        # Test service A directly
        @service_a.action("test_a")
        def handle_test_a():
            return service_a.response.send({"value": 42})
            
        # Create a message for service A
        message_a = {
            "action": "test_a",
            "payload": {"id": "test"},
            "reply_to": "reply_a"
        }
        
        # Send the message to service A
        service_a._process_message({"data": json.dumps(message_a)})
        
        # Verify the response
        self.assertIn("reply_a", results, "Service A should have sent a response")
        
        # Test service B directly
        @service_b.action("test_b")
        def handle_test_b():
            return service_b.response.send({"result": 84})
            
        # Create a message for service B
        message_b = {
            "action": "test_b",
            "payload": {"id": "test"},
            "reply_to": "reply_b"
        }
        
        # Send the message to service B
        service_b._process_message({"data": json.dumps(message_b)})
        
        # Verify the response
        self.assertIn("reply_b", results, "Service B should have sent a response")
        
        # Test service C directly
        @service_c.action("test_c")
        def handle_test_c():
            return service_c.response.send({"stored": True})
            
        # Create a message for service C
        message_c = {
            "action": "test_c",
            "payload": {"id": "test"},
            "reply_to": "reply_c"
        }
        
        # Send the message to service C
        service_c._process_message({"data": json.dumps(message_c)})
        
        # Verify the response
        self.assertIn("reply_c", results, "Service C should have sent a response")
        
        # Now test a simple end-to-end flow
        results.clear()  # Clear previous results
        
        # Set up proper handlers for the workflow
        @service_a.action("get_data")
        def a_handle_get_data():
            request_id = service_a.request.payload.get("id", "unknown")
            return service_a.response.send({
                "data": {"value": 42},
                "id": request_id
            })
            
        @service_b.action("process_data")
        def b_handle_process_data():
            request_id = service_b.request.payload.get("id", "unknown")
            value = service_b.request.payload.get("value", 0)
            result = value * 2
            return service_b.response.send({
                "result": result,
                "id": request_id
            })
            
        @service_c.action("store_result")
        def c_handle_store_result():
            request_id = service_c.request.payload.get("id", "unknown")
            result = service_c.request.payload.get("result", 0)
            return service_c.response.send({
                "stored": True,
                "id": request_id
            })
            
        # Create messages for a single workflow
        workflow_a = {
            "action": "get_data",
            "payload": {"id": "wf1"},
            "reply_to": "reply_wf1_a"
        }
        
        # Step 1: Process message A
        service_a._process_message({"data": json.dumps(workflow_a)})
        
        # Verify response from A
        self.assertIn("reply_wf1_a", results, "Service A should have sent a response")
        
        # Parse the response
        response_a = json.loads(results["reply_wf1_a"])
        self.assertEqual(response_a["payload"]["id"], "wf1", "Response should have the correct ID")
        
        # Get value from A's response
        value = response_a["payload"]["data"]["value"]
        
        # Step 2: Create and process message B
        workflow_b = {
            "action": "process_data",
            "payload": {"id": "wf1", "value": value},
            "reply_to": "reply_wf1_b"
        }
        
        service_b._process_message({"data": json.dumps(workflow_b)})
        
        # Verify response from B
        self.assertIn("reply_wf1_b", results, "Service B should have sent a response")
        
        # Parse the response
        response_b = json.loads(results["reply_wf1_b"])
        self.assertEqual(response_b["payload"]["id"], "wf1", "Response should have the correct ID")
        
        # Get result from B's response
        result = response_b["payload"]["result"]
        
        # Step 3: Create and process message C
        workflow_c = {
            "action": "store_result",
            "payload": {"id": "wf1", "result": result},
            "reply_to": "reply_wf1_c"
        }
        
        service_c._process_message({"data": json.dumps(workflow_c)})
        
        # Verify response from C
        self.assertIn("reply_wf1_c", results, "Service C should have sent a response")
        
        # Parse the response
        response_c = json.loads(results["reply_wf1_c"])
        self.assertEqual(response_c["payload"]["id"], "wf1", "Response should have the correct ID")
        
        # Verify the end-to-end data flow worked correctly
        self.assertEqual(result, value * 2, "Service B should have doubled the value")
        self.assertTrue(response_c["payload"]["stored"], "Service C should have confirmed storage")
        
        # Cleanup
        for service in [service_a, service_b, service_c]:
            service.thread_pool.shutdown(wait=True)


if __name__ == '__main__':
    unittest.main()
